/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 * 
 * Created on Nov 5, 2002
 */
package ca.uvic.csr.shrimp.gui.options;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.Properties;
import java.util.Vector;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import ca.uvic.csr.shrimp.LayoutConstants;
import ca.uvic.csr.shrimp.DisplayBean.DisplayBean;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpNode;
import ca.uvic.csr.shrimp.DisplayBean.layout.Layout;
import ca.uvic.csr.shrimp.ShrimpApplication.ApplicationAccessor;

/**
 * @author Nasir Rather
 *
 * Displays options for changing the default layout
 */
public class LayoutOptionsPanel extends JPanel implements ShrimpOptions {
	DisplayBean displayBean;
	Properties properties;

	// these represent the properites we want to ask user for
	//private String childrenOrSelection;
	private String layoutMode;
	private boolean resetLayout;
	private JPanel holder;

	public LayoutOptionsPanel(DisplayBean db) {
	
		displayBean = db;
		this.properties = ApplicationAccessor.getApplication().getProperties();
		holder = new JPanel();
	
		// get the previous settings from properties 
		resetLayout = false;
		layoutMode = properties.getProperty(DisplayBean.PROPERTY_KEY__DEFAULT_LAYOUT_MODE, DisplayBean.PROPERTY_DEFAULT_VALUE__DEFAULT_LAYOUT_MODE);
		//childrenOrSelection = properties.getProperty(DisplayBean.PROPERTY_KEY__CHILDREN_OR_SELECTION, ShrimpDisplayBean.PROPERTY_DEFAULT_VALUE__CHILDREN_OR_SELECTION);
	
		// get all available layouts
		Vector layouts = displayBean.getLayouts();
		Vector layoutNames = new Vector(layouts.size());
		for (Iterator iter = layouts.iterator(); iter.hasNext();) {
			Layout layout = (Layout) iter.next();	
			layoutNames.add(layout.getName());
		}
		// don't want users to choose the motion layout as the default
		layoutNames.remove(LayoutConstants.LAYOUT_MOTION);
		
		Collections.sort(layoutNames, String.CASE_INSENSITIVE_ORDER); // sort names alphabetically
	
		// Start making Gui ... M'm fun!! (hate gridbag)
		GridBagLayout gridBagLayout = new GridBagLayout();
		GridBagConstraints gridBagConstraints = new GridBagConstraints();
		
		holder.setLayout(gridBagLayout);
	
		gridBagConstraints.anchor = GridBagConstraints.WEST;
		gridBagConstraints.insets = new Insets(5, 5, 5, 5);
		// add label "1. Layout algorithm: "
		JLabel label = new JLabel("1. Default layout algorithm: ");
		gridBagLayout.setConstraints(label, gridBagConstraints);
		holder.add(label);
	
		gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
		// add the combobox
		final JComboBox comboBox = new JComboBox(layoutNames);
		comboBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				LayoutOptionsPanel.this.layoutMode = comboBox.getSelectedItem().toString();
			}
		});
		gridBagLayout.setConstraints(comboBox, gridBagConstraints);
		holder.add(comboBox);
		comboBox.setSelectedItem(layoutMode);
	
		gridBagConstraints.gridwidth = 2;
		gridBagConstraints.insets = new Insets(0, 25, 15, 5);
		gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
		// add the first checkbox
		final JCheckBox checkBox = new JCheckBox("Reset layouts of the nodes I have already visited in the current graph?");
		checkBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				resetLayout = checkBox.isSelected();
			}
		});
		gridBagLayout.setConstraints(checkBox, gridBagConstraints);
		holder.add(checkBox);
		
		/*
		gridBagConstraints.insets = new Insets(5, 5, 5, 5);
		gridBagConstraints.gridwidth = gridBagConstraints.REMAINDER;
		// add the label "2. Apply layout to:"
		JLabel label3 = new JLabel("2. Default apply layout to:");
		gridBagLayout.setConstraints(label3, gridBagConstraints);
		holder.add(label3);
	
		gridBagConstraints.gridwidth = gridBagConstraints.REMAINDER;
		gridBagConstraints.insets = new Insets(0, 25, 0, 5);
		// add first radio button
		JRadioButton radioButton = new JRadioButton("children of selected nodes");
		radioButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				LayoutOptionsPanel.this.childrenOrSelection = "children";
			}
		});
	
		gridBagLayout.setConstraints(radioButton, gridBagConstraints);
		holder.add(radioButton);
		// add second radiobutton
		JRadioButton radioButton2 = new JRadioButton("selected nodes");
		radioButton2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				LayoutOptionsPanel.this.childrenOrSelection = "selection";
			}
		});
		gridBagLayout.setConstraints(radioButton2, gridBagConstraints);
		holder.add(radioButton2);
	
		// create a button group for the radiobuttons
		final ButtonGroup buttonGroup = new ButtonGroup();
		buttonGroup.add(radioButton);
		buttonGroup.add(radioButton2);
	
		// enable one of the radiobuttons according to the previous settings
		if (childrenOrSelection.equalsIgnoreCase("selection")) {
			radioButton2.setSelected(true);
		} else {
			radioButton.setSelected(true);
		}
		*/
	
		gridBagConstraints.gridwidth = 2;
		gridBagConstraints.insets = new Insets(15, 5, 5, 5);
		gridBagConstraints.anchor = GridBagConstraints.CENTER;
	
		JPanel panel = new JPanel();
	
		gridBagLayout.setConstraints(panel, gridBagConstraints);
		holder.add(panel);
	
		this.add(holder, BorderLayout.NORTH);
	
		// put in center of screen
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int x = (int) (screen.getWidth() - this.getWidth()) / 2;
		int y = (int) (screen.getHeight() - this.getHeight()) / 2;
		setLocation(x, y);
	}

	private void applySettings() {
		// save settings to properties
		properties.setProperty(DisplayBean.PROPERTY_KEY__DEFAULT_LAYOUT_MODE, layoutMode);
		//properties.setProperty(ShrimpDisplayBean.PROPERTY_KEY__CHILDREN_OR_SELECTION, childrenOrSelection);

		if (resetLayout) {
			Collection rootNodes = displayBean.getDataDisplayBridge().getRootNodes();
			for (Iterator iter = rootNodes.iterator(); iter.hasNext();) {
                ShrimpNode rootNode = (ShrimpNode) iter.next();
    			displayBean.setLayoutMode(displayBean.getDataDisplayBridge().getDescendentNodes(rootNode, false), layoutMode, false, false);
            }
		}
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.gui.options.ShrimpOptions#ok()
	 */
	public void ok() {
		applySettings();		
	}
	
	/**
	 * @see ca.uvic.csr.shrimp.gui.options.ShrimpOptions#cancel()
	 */
	public void cancel() {
	}
}
